'use strict';

exports.install = install;

var _commander = require('commander');

var commander = _commander;

var _expandTilde = require('expand-tilde');

var expandTilde = _expandTilde;

var _package = require('../package');

var npmPackage = _package;

var _chalk = require('chalk');

var chalk = _chalk;

/**
 * Perform install
 * @param  {array<string>} orgRepos - list of GitHub organization repositories
 *                                  of the format "tool!organization/repository".
 *                                  i.e. "pip!ipython/ipython" for
 *                                  github.com/ipython/ipython to be installed
 *                                  by pip.
 *                                  Possible tools include pip and npm.
 * @param  {array<string>} argv - from process.argv
 * @return {[type]}          [description]
 */

function install(orgRepos, argv) {

    // Ordered list of install steps
    var steps = [require('./steps/filter'), require('./steps/check-ssl'), require('./steps/check-github'), require('./steps/check-system'), require('./steps/check-destination'), require('./steps/config'), require('./steps/mk-destination'), require('./steps/clone'), require('./steps/npm'), require('./steps/pip')];

    // Contruct steps
    steps = steps.map(function (step) {
        return new step['default'](commander);
    });

    // Parse args
    commander.option('-s, --silent', 'don\'t prompt the user for anything');
    commander.version(npmPackage.version);
    commander.usage('[options] <githubName> <installdir>');
    commander.parse(argv);

    // Print header
    console.log(chalk.bold.cyan('Jupyter dev install tool'));
    console.log(chalk.bold.cyan('Wizard for setting up a Jupyter development environment'));
    if (commander.args.length < 2) {
        console.error('Not enough arguments');
        process.exit(1);
    }

    // Add globals
    commander.orgRepos = orgRepos.map(function (x) {
        return x.split(':')[1];
    });
    commander.repos = commander.orgRepos.map(function (x) {
        return x.split('/')[1];
    });
    commander.orgs = commander.orgRepos.map(function (x) {
        return x.split('/')[0];
    }).filter(function (x, i, self) {
        return self.indexOf(x) === i;
    });
    commander.githubName = commander.args[0];
    commander.installdir = expandTilde(commander.args[1]);

    // Separate repos based on install tool
    var orgRepoTools = orgRepos.map(function (x) {
        return x.split(':');
    });
    var tools = orgRepoTools.map(function (x) {
        return x[0];
    }).filter(function (x, i, self) {
        return self.indexOf(x) === i;
    });
    commander.tools = {};
    tools.forEach(function (tool) {
        commander.tools[tool] = orgRepoTools.filter(function (x) {
            return x[0] === tool;
        }).map(function (x) {
            return x[1];
        });
    });

    // Run each step sequentially
    var results = Promise.resolve();
    steps.forEach(function (step) {
        results = results.then(function (previous) {
            return step.tryRun(previous);
        })['catch'](function (error) {
            console.error('Error while executing ' + step.constructor.name, error, error.stack);
            process.exit(1);
        });
    });
    return results;
}